@@ -51,6 +51,8 @@ class P_cif(StructureParser):
                    unit in the CIF file
     labelindex  -- dictionary mapping unique atom label to index of atom
                    in self.asymmetric_unit
+    anisotropy  -- dictionary mapping unique atom label to displacement
+                   anisotropy resolved at that site
     cif_sgname  -- space group name obtained by looking up the value of
                    _space_group_name_Hall, _symmetry_space_group_name_Hall,
                    _space_group_name_H-M_alt, _symmetry_space_group_name_H-M
@@ -241,6 +243,7 @@ class P_cif(StructureParser):
         self.eau = None
         self.asymmetric_unit = None
         self.labelindex = {}
+        self.anisotropy = {}
         self.cif_sgname = None
         pass
 
@@ -278,8 +281,7 @@ class P_cif(StructureParser):
         """
         self.ciffile = None
         self.filename = filename
-        fileurl = _fixIfWindowsPath(filename)
-        rv = self._parseCifDataSource(fileurl)
+        rv = self._parseCifDataSource(filename)
         # all good here
         return rv
 
@@ -309,7 +311,9 @@ class P_cif(StructureParser):
         self.stru = None
         try:
             with _suppressCifParserOutput():
-                self.ciffile = CifFile(datasource)
+                # Use `grammar` option to digest values with curly-brackets.
+                # Ref: https://bitbucket.org/jamesrhester/pycifrw/issues/19
+                self.ciffile = CifFile(datasource, grammar='auto')
                 for blockname in self.ciffile.keys():
                     self._parseCifBlock(blockname)
                     # stop after reading the first structure
@@ -336,6 +340,7 @@ class P_cif(StructureParser):
         # here block contains structure, initialize output data
         self.stru = Structure()
         self.labelindex.clear()
+        self.anisotropy.clear()
         # execute specialized block parsers
         self._parse_lattice(block)
         self._parse_atom_site_label(block)
@@ -383,6 +388,8 @@ class P_cif(StructureParser):
         """
         # process _atom_site_label
         atom_site_loop = block.GetLoop('_atom_site_label')
+        does_adp_type = ('_atom_site_adp_type' in atom_site_loop or
+                         '_atom_site_thermal_displace_type' in atom_site_loop)
         # get a list of setters for atom_site values
         prop_setters = P_cif._get_atom_setters(atom_site_loop)
         # index of the _atom_site_label item for the labelindex dictionary
@@ -399,6 +406,8 @@ class P_cif(StructureParser):
             a = self.stru.getLastAtom()
             for fset, val in zip(prop_setters, values):
                 fset(a, val)
+            if does_adp_type:
+                self.anisotropy[curlabel] = a.anisotropy
         return
 
 
@@ -412,8 +421,6 @@ class P_cif(StructureParser):
         No return value.
         """
         if '_atom_site_aniso_label' not in block: return
-        # was anisotropy processed in the _atom_site_label loop?
-        isotropy_done = _hasAtomSiteADPType(block)
         # something to do here:
         adp_loop = block.GetLoop('_atom_site_aniso_label')
         # index of the _atom_site_label column
@@ -422,10 +429,14 @@ class P_cif(StructureParser):
         prop_setters = P_cif._get_atom_setters(adp_loop)
         sitedatalist = zip(*adp_loop.values())
         for values in sitedatalist:
-            idx = self.labelindex[values[ilb]]
+            lb = values[ilb]
+            if lb == '?':
+                break
+            idx = self.labelindex[lb]
             a = self.stru[idx]
-            if not isotropy_done:
+            if not lb in self.anisotropy:
                 a.anisotropy = True
+                self.anisotropy[lb] = True
             for fset, val in zip(prop_setters, values):
                 fset(a, val)
         return
@@ -463,8 +474,8 @@ class P_cif(StructureParser):
         sg_nameHM = (block.get('_space_group_name_H-M_alt', '') or
                 block.get('_symmetry_space_group_name_H-M', ''))
         self.cif_sgname = (sg_nameHall or sg_nameHM or None)
-        sgid = (int(block.get('_space_group_IT_number', '0')) or
-                int(block.get('_symmetry_Int_Tables_number', '0')) or
+        sgid = (block.get('_space_group_IT_number', '') or
+                block.get('_symmetry_Int_Tables_number', '') or
                 sg_nameHM)
         self.spacegroup = None
         # try to reuse existing space group from symmetry operations
@@ -511,12 +522,11 @@ class P_cif(StructureParser):
         self.eau = ExpandAsymmetricUnit(self.spacegroup, corepos, coreUijs,
                                         eps=self.eps)
         # setup anisotropy according to symmetry requirements
-        # was isotropy flag already processed
-        isotropy_done = (_hasAtomSiteADPType(block) or
-                         '_atom_site_aniso_label' in block)
-        if not isotropy_done:
-            for ca, uisotropy in zip(self.stru, self.eau.Uisotropy):
+        # unless it was already explicitly set
+        for ca, uisotropy in zip(self.stru, self.eau.Uisotropy):
+            if not ca.label in self.anisotropy:
                 ca.anisotropy = not uisotropy
+                self.anisotropy[ca.label] = ca.anisotropy
         # build a nested list of new atoms:
         newatoms = []
         for i, ca in enumerate(self.stru):
@@ -694,22 +704,6 @@ def getSymOp(s):
     rv = SymOp(R, t)
     return rv
 
-
-def _fixIfWindowsPath(filename):
-    """Convert Windows-style path to valid local URL.
-    CifFile loads files using urlopen, which fails for Windows-style paths.
-
-    filename -- path to be fixed
-
-    Return fixed URL when run on Windows, otherwise return filename.
-    """
-    rv = filename
-    if filename[:1].isalpha() and filename[1:3] == ':\\':
-        from urllib.request import pathname2url
-        rv = pathname2url(filename)
-    return rv
-
-
 def getParser(eps=None):
     """Return new parser object for CIF structure format.
 
@@ -734,12 +728,3 @@ def _suppressCifParserOutput():
     finally:
         yapps3_compiled_rt.print_error = print_error
     pass
-
-
-def _hasAtomSiteADPType(block):
-    """Return True if the CIF specifies _atom_site_adp_type.
-    """
-    atom_site_loop = block.GetLoop('_atom_site_label')
-    rv = ('_atom_site_adp_type' in atom_site_loop or
-          '_atom_site_thermal_displace_type' in atom_site_loop)
-    return rv
